<?php
/**
 * Mondial Relay Official WooCommerce Shipping settings
 *
 * @package mondialrelay
 */
declare(strict_types=1);

defined( 'ABSPATH' ) || exit;

/**
 * WooCommerce Shipping settings tab
 */
class MRWP_Shipping_Settings
{
    static $section_name = 'mrwp-shipping';

    /**
     * Constructor
     */
    public function __construct()
    {
        add_action( 'woocommerce_shipping_zone_after_methods_table', array( $this, 'add_duplicate_script' ) );
        add_action( 'admin_post_mrwp-duplicate-shipping',  array( $this, 'action_duplicate' ) );

        // add_filter( 'woocommerce_get_sections_shipping', array( $this, 'add_section' ) );
        // add_filter( 'woocommerce_get_settings_shipping', array( $this, 'add_settings' ), 10, 2 );
        // add_action( 'wp_ajax_mrwp_export_zones', array( $this, 'export_zones' ) );
    }


    /**
     * Export shipping zones config as JSON
     *
     * This is invoked as an AJAX action
     */
    public function add_duplicate_script()
    {
        global $wpdb;

        // Our methods from woocommerce_shipping_zone_methods
        $sql = "SELECT *
        FROM {$wpdb->prefix}woocommerce_shipping_zone_methods WHERE method_id = 'mondialrelay_official_shipping'";
        $methods = $wpdb->get_results( $sql );

        $instance_ids = array_map( [__CLASS__, 'get_instance_id' ], $methods );

        wp_register_script( 'mondialrelay_shipping', plugin_dir_url( __DIR__ ) . 'admin/js/MRWP-shipping.js', array( 'jquery' ), class_MRWP_main::VERSION );
        wp_enqueue_script( 'mondialrelay_shipping' );
        wp_localize_script(
            'mondialrelay_shipping',
            'mrwp_shipping',
            [
                'instance_ids' => $instance_ids,
                'label'        => __( 'Duplicate', 'mondialrelay-wordpress' ),
                'base_url'     => wp_nonce_url(
                    add_query_arg( 'action', 'mrwp-duplicate-shipping', admin_url( 'admin-post.php' ) ),
                    'mrwp-duplicate-shipping'
                ),
            ]
        );
    }


    /**
     * Export shipping zones config as JSON
     *
     * This is invoked as an AJAX action
     */
    public function action_duplicate()
    {
        check_admin_referer( 'mrwp-duplicate-shipping' );

        // Check action
        if ( ! isset( $_GET['action'] ) || $_GET['action'] != 'mrwp-duplicate-shipping' ) {
            return;
        }

        // Check permissions
        if ( ! current_user_can( 'manage_woocommerce' ) ) {
            return;
        }

        $instance_id = (int)$_GET[ 'instance_id' ] ?? 0 ;

        if ( ! $instance_id ) {
            wp_die( __( 'Shipping method duplication error.', 'mondialrelay-wordpress' ) );

            return;
        }

        // Retrieve shipping method
        $method = WC_Shipping_Zones::get_shipping_method( $instance_id );

        // Update method title
        $method->instance_settings['title'] = $method->title .= ' (' . __( 'copy', 'woocommerce' ) . ')';

        // Get the zone
        $zone = WC_Shipping_Zones::get_zone_by( 'instance_id', $instance_id );

        if ( $zone ) {
            $instance_id      = $zone->add_shipping_method( $method->id );
            $method->instance_id = $instance_id;

            update_option( $method->get_instance_option_key(), $method->instance_settings, 'yes' );
        }

        wp_redirect( wp_get_referer() );
        exit;
    }

    /**
     * Export shipping zones config as JSON
     *
     * This is invoked as an AJAX action
     */
    public function export_zones()
    {
        global $wpdb;

        // Methods from woocommerce_shipping_zone_methods
        $sql = "SELECT *
                FROM {$wpdb->prefix}woocommerce_shipping_zone_methods WHERE method_id = 'mondialrelay_official_shipping'";
        $methods = $wpdb->get_results( $sql );

        // Build zone ids list for further queriess
        $zone_ids = array_map( [__CLASS__, 'get_zone_id' ], $methods );
        $zone_list = implode( ',', $zone_ids );

        // Zones from woocommerce_shipping_zones
        $sql = "SELECT *
                FROM {$wpdb->prefix}woocommerce_shipping_zones WHERE zone_id IN( " . $zone_list . ")";
        $zones = $wpdb->get_results( $sql );

        // Locations from woocommerce_shipping_zone_locations
        $sql = "SELECT *
                FROM {$wpdb->prefix}woocommerce_shipping_zone_locations WHERE zone_id IN( " . $zone_list . ")";
        $locations = $wpdb->get_results( $sql );

        // woocommerce_mondialrelay_official_shipping_36_settings
        $instance_ids = array_map( [__CLASS__, 'get_instance_id' ], $methods );
        $option_settings = array();
        foreach ( $instance_ids as $instance ) {
            $option_name       = sprintf( 'woocommerce_mondialrelay_official_shipping_%d_settings', $instance );
            $option_settings[] = get_option( $option_name, false );
        }

        wp_send_json( array(
            'signature' => 'mrwp-export-shipping-config',
            'locations' => $locations,
            'methods'   => $methods,
            'options'   => $option_settings,
            'zones'     => $zones,
        ) );
    }

    /**
     * Add section to WooCommerce Settings Shipping tab
     *
     * This is invoked as a filter
     */
    public function add_section( $sections )
    {
        $sections[ self::$section_name ] = __( 'Mondial Relay', 'mondialrelay-wordpress' );

        return $sections;
    }

    /**
     * Add content to the section
     *
     * This is invoked as a filter
     */
    public function add_settings( $settings, $current_section )
    {
        if ( $current_section == self::$section_name ) {

            $settings = array();

            // Title
            $settings[] = array(
                'name' => __( 'Mondial Relay', 'mondialrelay-wordpress' ),
                'type' => 'title',
                'desc' => __( 'Whatever explanation', 'mondialrelay-wordpress' ),
                'id'   => 'mrwp-shipping',
            );

            // checkbox option
            $settings[] = array(
                'css'      => 'min-width:300px;',
                'desc'     => __( 'Option desc', 'mondialrelay-wordpress' ),
                'desc_tip' => __( 'Option desc tip', 'mondialrelay-wordpress' ),
                'id'       => self::$section_name . 'option-id',
                'name'     => __( 'Option name', 'mondialrelay-wordpress' ),
                'type'     => 'checkbox',
            );

            $settings[] = array(
                'type' => 'sectionend',
                'id'   =>  self::$section_name,
            );
        }

        return $settings;
    }

    /**
     * Returns zone_id property from object
     */
    private static function get_zone_id( $object ): int
    {
        return (int)$object->zone_id;
    }

    /**
     * Returns instance_id property from object
     */
    private static function get_instance_id( $object ): int
    {
        return (int)$object->instance_id;
    }
}